from fileformats.core.mixin import WithMagicNumber
from fileformats.generic import BinaryFile


# Document formats
class Document(BinaryFile):
    # iana_mime = None
    pass


class Pdf(WithMagicNumber, Document):
    ext = ".pdf"
    magic_number = b"%PDF-"
    binary = True

    iana_mime = "application/pdf"


class Msword(Document):
    ext = ".doc"
    binary = True

    iana_mime = "application/msword"


class Postscript(WithMagicNumber, Document):
    ext = ".eps"
    alternate_exts = (".ps",)
    magic_number = b"%!"
    iana_mime = "application/postscript"


__all__ = ["Document", "Pdf", "Msword", "Postscript"]



from fileformats.core import __version__

from .abstract import (
    AbstractFile,
    AbstractSubclass,
    AnotherConcreteClass,
    ConcreteClass,
    ConvertibleToFile,
)
from .basic import Bar, Baz, Foo, Qux
from .classifiers.file import A, B, C, D, E, F, G, H, J, K, L, M, N, P, Q, R, TestField
from .classifiers.generic import Classified, U, V, W, X, Y, Z
from .headers import (
    EncodedText,
    ImageWithHeader,
    MyBinaryFormat,
    MyBinaryFormatX,
    MyFormat,
    MyFormatGz,
    MyFormatGzX,
    MyFormatX,
    MyHeader,
    MyOtherBinaryFormatX,
    SeparateHeader,
    Xyz,
    YFile,
    YourFormat,
    ZFile,
)
from .magic import Magic, MagicVersion

__all__ = [
    "__version__",
    "Foo",
    "Bar",
    "Baz",
    "Qux",
    "YFile",
    "ZFile",
    "Xyz",
    "MyFormat",
    "MyFormatGz",
    "MyFormatGzX",
    "MyFormatX",
    "MyBinaryFormat",
    "MyHeader",
    "MyBinaryFormatX",
    "MyOtherBinaryFormatX",
    "YourFormat",
    "SeparateHeader",
    "ImageWithHeader",
    "EncodedText",
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "H",
    "J",
    "K",
    "L",
    "M",
    "N",
    "P",
    "Q",
    "R",
    "TestField",
    "Classified",
    "U",
    "V",
    "W",
    "X",
    "Y",
    "Z",
    "Magic",
    "MagicVersion",
    "AbstractFile",
    "AbstractSubclass",
    "ConcreteClass",
    "AnotherConcreteClass",
    "ConvertibleToFile",
]



import typing as ty

from .decorators import classproperty
from .exceptions import FormatDefinitionError


class Classifier:
    """Base class for all file-format "classifiers", including datatypes and abstract
    types"""

    @classproperty  # type: ignore[arg-type]
    def type_name(cls) -> str:
        """Name of type to be used in __repr__. Defined here so it can be overridden"""
        return cls.__name__  # type: ignore

    @classproperty  # type: ignore[arg-type]
    def namespace(cls) -> ty.Optional[str]:
        """The "namespace" the format belongs to under the "fileformats" umbrella
        namespace"""
        module_parts = cls.__module__.split(".")
        if module_parts[0] != "fileformats":
            raise FormatDefinitionError(
                f"Cannot determine namespace for {cls} format as it is not in the "
                "fileformats namespace package"
            )
        namespace = module_parts[1]
        if namespace == "vendor":
            if len(module_parts) < 4:
                raise FormatDefinitionError(
                    f"Cannot determine namespace for vendor-specific format, {cls} it needs "
                    "to be in a subpackage of the form `fileformats.vendor.<vendor-name>.<namespace>`,"
                    f"found `{'.'.join(module_parts)}`"
                )
            namespace = module_parts[3]
        return namespace.replace("_", "-")

    @classproperty
    def vendor(cls) -> ty.Optional[str]:
        module_parts = cls.__module__.split(".")
        if module_parts[0] != "fileformats" or module_parts[1] != "vendor":
            return None
        return module_parts[2].replace("_", "-")

    def dummy(self) -> float:

        i: int = 0

        return i



from fileformats.application import Presentation, Zip


class Presentationml_Presentation(Zip, Presentation):
    ext = ".pptx"


class Presentationml_Template(Zip, Presentation):
    ext = ".potx"


class Presentationml_Slideshow(Zip, Presentation):
    ext = ".ppsx"



from fileformats.application import Presentation, Zip


class MsPowerPoint_Presentation_MacroEnabled_12(Zip, Presentation):
    ext = ".pptm"
    mime = "application/vnd.ms-powerpoint.presentation.macroEnabled.12"


class MsPowerPoint_Template_MacroEnabled_12(Zip, Presentation):
    ext = ".potm"
    mime = "application/vnd.ms-powerpoint.template.macroEnabled.12"


class MsPowerPoint_Slideshow_MacroEnabled_12(Zip, Presentation):
    ext = ".ppsm"
    mime = "application/vnd.ms-powerpoint.slideshow.macroEnabled.12"



from .ascii import CacheManifest
from .base import Text
from .unicode import (
    N3,
    T140,
    Css,
    Csv,
    CsvSchema,
    Dns,
    Encaprtp,
    Fhirpath,
    Flexfec,
    Fwdred,
    Gff3,
    GrammarRefList,
    Hl7v2,
    Javascript,
    JcrCnd,
    Mizar,
    Parameters,
    Parityfec,
    Plain,
    ProvenanceNotation,
    Prs_Fallenstein_Rst,
    Prs_Lines_Tag,
    Prs_Prop_Logic,
    Raptorfec,
    Red,
    Rfc822Headers,
    RichText,
    RtpEncAescm128,
    Rtploopback,
    Rtx,
    Sgml,
    Shaclc,
    Shex,
    Spdx,
    Strings,
    TextFile,
    Troff,
    Turtle,
    Ulpfec,
    UriList,
    Vcard,
    Vtt,
    Wgsl,
    XmlExternalParsedEntity,
)

from fileformats.application import (  # isort: skip
    Json,
    Xml,
    Yaml,
)

__all__ = [
    "__version__",
    "Text",
    "Plain",
    "TextFile",
    "Csv",
    "Tsv",
    "Html",
    "Markdown",
    "RestructedText",
    "_1d_interleaved_parityfec",
    "CacheManifest",
    "Calendar",
    "Cql",
    "CqlExpression",
    "CqlIdentifier",
    "Css",
    "CsvSchema",
    "Dns",
    "Encaprtp",
    "Fhirpath",
    "Flexfec",
    "Fwdred",
    "Gff3",
    "GrammarRefList",
    "Hl7v2",
    "Javascript",
    "JcrCnd",
    "Json",
    "Mizar",
    "N3",
    "Parameters",
    "Parityfec",
    "ProvenanceNotation",
    "Prs_Fallenstein_Rst",
    "Prs_Lines_Tag",
    "Prs_Prop_Logic",
    "Raptorfec",
    "Red",
    "Rfc822Headers",
    "RichText",
    "RtpEncAescm128",
    "Rtploopback",
    "Rtx",
    "Sgml",
    "Shaclc",
    "Shex",
    "Spdx",
    "Strings",
    "T140",
    "Troff",
    "Turtle",
    "Ulpfec",
    "UriList",
    "Vcard",
    "Vtt",
    "Wgsl",
    "Xml",
    "XmlExternalParsedEntity",
    "Yaml",
]



from fileformats.vendor.openxmlformats_officedocument.application import (
    Wordprocessingml_Document,
)


class MsWord_Document_MacroEnabled_12(Wordprocessingml_Document):
    ext = ".docm"
    mime = "application/vnd.ms-word.document.macroEnabled.12"


class MsWord_Template_MacroEnabled_12(Wordprocessingml_Document):
    ext = ".dotm"
    mime = "application/vnd.ms-word.template.macroEnabled.12"



from fileformats.core.mixin import WithMagicNumber
from fileformats.generic import BinaryFile

from .base import Text


class CacheManifest(WithMagicNumber, Text, BinaryFile):
    """"""

    iana_mime = "text/cache-manifest"
    ext = ".appcache"
    alternate_exts = ('"manifest"',)
    magic_number = b"CACHE MANIFEST"



from fileformats.application import Spreadsheet, Zip


class Sheet_MacroEnabled_12(Zip, Spreadsheet):
    ext = ".xlsm"
    mime = "application/vnd.ms-excel.sheet.macroEnabled.12"


class Template_MacroEnabled_12(Zip, Spreadsheet):
    ext = ".xltm"
    mime = "application/vnd.ms-excel.template.macroEnabled.12"


class Binary_MacroEnabled_12(Zip, Spreadsheet):
    ext = ".xlsb"
    mime = "application/vnd.ms-excel.sheet.binary.macroEnabled.12"



from fileformats.application import Spreadsheet, Zip


class Spreadsheetml_Sheet(Zip, Spreadsheet):
    ext = ".xlsx"
    mime = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"


class Spreadsheetml_Template(Zip, Spreadsheet):
    ext = ".xltx"
    mime = "application/vnd.openxmlformats-officedocument.spreadsheetml.template"



from .document import Wordprocessingml_Document, Wordprocessingml_Template
from .helper import (
    Drawingml_Chart___Xml,
    Drawingml_Diagram___Xml,
    Presentationml_HandoutMaster___Xml,
    Presentationml_NotesMaster___Xml,
    Presentationml_NotesSlide___Xml,
    Presentationml_Slide___Xml,
    Presentationml_SlideLayout___Xml,
    Presentationml_SlideMaster___Xml,
    Theme___Xml,
    ThemeOverride___Xml,
    VmlDrawing,
    Wordprocessingml_Comments___Xml,
    Wordprocessingml_Endnotes___Xml,
    Wordprocessingml_Footer___Xml,
    Wordprocessingml_Footnotes___Xml,
    Wordprocessingml_Header___Xml,
)
from .presentation import (
    Presentationml_Presentation,
    Presentationml_Slideshow,
    Presentationml_Template,
)
from .spreadsheet import Spreadsheetml_Sheet, Spreadsheetml_Template

__all__ = [
    "Wordprocessingml_Document",
    "Wordprocessingml_Template",
    "Presentationml_Presentation",
    "Presentationml_Slideshow",
    "Presentationml_Template",
    "Spreadsheetml_Sheet",
    "Spreadsheetml_Template",
    "Drawingml_Chart___Xml",
    "Drawingml_Diagram___Xml",
    "Presentationml_HandoutMaster___Xml",
    "Presentationml_NotesMaster___Xml",
    "Presentationml_NotesSlide___Xml",
    "Presentationml_Slide___Xml",
    "Presentationml_SlideLayout___Xml",
    "Presentationml_SlideMaster___Xml",
    "Theme___Xml",
    "ThemeOverride___Xml",
    "VmlDrawing",
    "Wordprocessingml_Comments___Xml",
    "Wordprocessingml_Endnotes___Xml",
    "Wordprocessingml_Footer___Xml",
    "Wordprocessingml_Footnotes___Xml",
    "Wordprocessingml_Header___Xml",
]



from fileformats.application import Xml


class Drawingml_Chart___Xml(Xml):
    ext = ".chart"
    mime = "application/vnd.openxmlformats-officedocument.drawingml.chart+xml"


class Drawingml_Diagram___Xml(Xml):
    ext = ".diagram"
    mime = "application/vnd.openxmlformats-officedocument.drawingml.diagram+xml"


class Wordprocessingml_Footnotes___Xml(Xml):
    ext = ".footnotes"
    mime = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml"
    )


class Wordprocessingml_Endnotes___Xml(Xml):
    ext = ".endnotes"
    mime = "application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml"


class Wordprocessingml_Header___Xml(Xml):
    ext = ".header"
    mime = "application/vnd.openxmlformats-officedocument.wordprocessingml.header+xml"


class Wordprocessingml_Footer___Xml(Xml):
    ext = ".footer"
    mime = "application/vnd.openxmlformats-officedocument.wordprocessingml.footer+xml"


class Wordprocessingml_Comments___Xml(Xml):
    ext = ".comments"
    mime = "application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml"


class Presentationml_Slide___Xml(Xml):
    ext = ".sldx"
    mime = "application/vnd.openxmlformats-officedocument.presentationml.slide+xml"


class Presentationml_SlideLayout___Xml(Xml):
    ext = ".sldlayout"
    mime = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideLayout+xml"
    )


class Presentationml_SlideMaster___Xml(Xml):
    ext = ".sldmaster"
    mime = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideMaster+xml"
    )


class Presentationml_NotesSlide___Xml(Xml):
    ext = ".sldnotes"
    mime = "application/vnd.openxmlformats-officedocument.presentationml.notesSlide+xml"


class Presentationml_HandoutMaster___Xml(Xml):
    ext = ".handoutmaster"
    mime = (
        "application/vnd.openxmlformats-officedocument.presentationml.handoutMaster+xml"
    )


class Presentationml_NotesMaster___Xml(Xml):
    ext = ".notesmaster"
    mime = (
        "application/vnd.openxmlformats-officedocument.presentationml.notesMaster+xml"
    )


class Theme___Xml(Xml):
    ext = ".thmx"
    mime = "application/vnd.openxmlformats-officedocument.theme+xml"


class ThemeOverride___Xml(Xml):
    ext = ".thmoverride"
    mime = "application/vnd.openxmlformats-officedocument.themeOverride+xml"


class VmlDrawing(Xml):
    ext = ".vml"
    mime = "application/vnd.openxmlformats-officedocument.vmlDrawing"



from fileformats.core import __version__

from .archive import Archive, Bzip, Gzip, Tar, TarGzip, Zip
from .document import Document, Msword, Pdf, Postscript
from .medical import Dicom
from .misc import (
    H224,
    P21,
    A2l,
    Ace___Cbor,
    Ace___Json,
    Activemessage,
    Activity___Json,
    Aif___Cbor,
    Aif___Json,
    AltoCdni___Json,
    AltoCdnifilter___Json,
    AltoCostmap___Json,
    AltoCostmapfilter___Json,
    AltoDirectory___Json,
    AltoEndpointcost___Json,
    AltoEndpointcostparams___Json,
    AltoEndpointprop___Json,
    AltoEndpointpropparams___Json,
    AltoError___Json,
    AltoNetworkmap___Json,
    AltoNetworkmapfilter___Json,
    AltoPropmap___Json,
    AltoPropmapparams___Json,
    AltoUpdatestreamcontrol___Json,
    AltoUpdatestreamparams___Json,
    Aml,
    AndrewInset,
    Applefile,
    At___Jwt,
    Atf,
    Atfx,
    Atom___Xml,
    Atomcat___Xml,
    Atomdeleted___Xml,
    Atomicmail,
    Atomsvc___Xml,
    AtscDwd___Xml,
    AtscDynamicEventMessage,
    AtscHeld___Xml,
    AtscRdt___Json,
    AtscRsat___Xml,
    Atxml,
    AuthPolicy___Xml,
    AutomationmlAml___Xml,
    AutomationmlAmlx___Zip,
    BacnetXdd___Zip,
    BatchSmtp,
    Beep___Xml,
    Calendar___Json,
    Calendar___Xml,
    CallCompletion,
    Cals_1840,
    Captive___Json,
    Cbor,
    CborSeq,
    Cccex,
    Ccmp___Xml,
    Ccxml___Xml,
    Cda___Xml,
    Cdfx___Xml,
    CdmiCapability,
    CdmiContainer,
    CdmiDomain,
    CdmiObject,
    CdmiQueue,
    Cdni,
    Cea,
    Cea_2018___Xml,
    Cellml___Xml,
    Cfw,
    City___Json,
    Clr,
    Clue___Xml,
    ClueInfo___Xml,
    Cms,
    Cnrp___Xml,
    CoapGroup___Json,
    CoapPayload,
    Commonground,
    ConciseProblemDetails___Cbor,
    ConferenceInfo___Xml,
    Cose,
    CoseKey,
    CoseKeySet,
    CoseX509,
    Cpl___Xml,
    Csrattrs,
    Csta___Xml,
    Cstadata___Xml,
    Csvm___Json,
    Cwl,
    Cwl___Json,
    Cwt,
    Cybercash,
    Dash___Xml,
    Dashdelta,
    DashPatch___Xml,
    Davmount___Xml,
    DcaRft,
    Dcd,
    DecDx,
    DialogInfo___Xml,
    Dicom___Json,
    Dicom___Xml,
    Dii,
    Dit,
    Dns,
    Dns___Json,
    DnsMessage,
    Dots___Cbor,
    Dpop___Jwt,
    Dskpp___Xml,
    Dssc___Der,
    Dssc___Xml,
    Dvcs,
    EdiConsent,
    Edifact,
    EdiX12,
    Efi,
    Elm___Json,
    Elm___Xml,
    Emergencycalldata_Cap___Xml,
    Emergencycalldata_Comment___Xml,
    Emergencycalldata_Control___Xml,
    Emergencycalldata_Deviceinfo___Xml,
    Emergencycalldata_Ecall_Msd,
    Emergencycalldata_Legacyesn___Json,
    Emergencycalldata_Providerinfo___Xml,
    Emergencycalldata_Serviceinfo___Xml,
    Emergencycalldata_Subscriberinfo___Xml,
    Emergencycalldata_Veds___Xml,
    Emma___Xml,
    Emotionml___Xml,
    Encaprtp,
    Epp___Xml,
    Epub___Zip,
    Eshop,
    Exi,
    ExpectCtReport___Json,
    Express,
    Fastinfoset,
    Fastsoap,
    Fdf,
    Fdt___Xml,
    Fhir___Json,
    Fhir___Xml,
    Fits,
    Flexfec,
    FrameworkAttributes___Xml,
    Geo___Json,
    Geo___JsonSeq,
    Geopackage___Sqlite3,
    Geoxacml___Xml,
    GltfBuffer,
    Gml___Xml,
    Held___Xml,
    Hl7v2___Xml,
    Http,
    Hyperstudio,
    IbeKeyRequest___Xml,
    IbePkgReply___Xml,
    IbePpData,
    Iges,
    ImIscomposing___Xml,
    Index,
    Index_Cmd,
    Index_Obj,
    Index_Response,
    Index_Vnd,
    Inkml___Xml,
    Iotp,
    Ipfix,
    Ipp,
    Isup,
    Its___Xml,
    JavaArchive,
    Jf2feed___Json,
    Jose,
    Jose___Json,
    Jrd___Json,
    Jscalendar___Json,
    JsonPatch___Json,
    JsonSeq,
    Jwk___Json,
    JwkSet___Json,
    Jwt,
    KpmlRequest___Xml,
    KpmlResponse___Xml,
    Ld___Json,
    Lgr___Xml,
    LinkFormat,
    Linkset,
    Linkset___Json,
    LoadControl___Xml,
    Logout___Jwt,
    Lost___Xml,
    Lostsync___Xml,
    Lpf___Zip,
    Lxf,
    MacBinhex40,
    Macwriteii,
    Mads___Xml,
    Manifest___Json,
    Marc,
    Marcxml___Xml,
    Mathematica,
    Mathml___Xml,
    MathmlContent___Xml,
    MathmlPresentation___Xml,
    MbmsAssociatedProcedureDescription___Xml,
    MbmsDeregister___Xml,
    MbmsEnvelope___Xml,
    MbmsMsk___Xml,
    MbmsMskResponse___Xml,
    MbmsProtectionDescription___Xml,
    MbmsReceptionReport___Xml,
    MbmsRegister___Xml,
    MbmsRegisterResponse___Xml,
    MbmsSchedule___Xml,
    MbmsUserServiceDescription___Xml,
    Mbox,
    MediaControl___Xml,
    MediaPolicyDataset___Xml,
    Mediaservercontrol___Xml,
    MergePatch___Json,
    Metalink4___Xml,
    Mets___Xml,
    Mf4,
    Mikey,
    Mipc,
    MissingBlocks___CborSeq,
    MmtAei___Xml,
    MmtUsd___Xml,
    Mods___Xml,
    MosskeyData,
    MosskeyRequest,
    MossKeys,
    MossSignature,
    Mp4,
    Mp21,
    Mpeg4Generic,
    Mpeg4Iod,
    Mpeg4IodXmt,
    MrbConsumer___Xml,
    MrbPublish___Xml,
    MscIvr___Xml,
    MscMixer___Xml,
    Mud___Json,
    MultipartCore,
    Mxf,
    Nasdata,
    NewsCheckgroups,
    NewsGroupinfo,
    NewsTransmission,
    Nlsml___Xml,
    Node,
    NQuads,
    Nss,
    NTriples,
    OauthAuthzReq___Jwt,
    ObliviousDnsMessage,
    OcspRequest,
    OcspResponse,
    OctetStream,
    Oda,
    Odm___Xml,
    Odx,
    OebpsPackage___Xml,
    Ogg,
    OhttpKeys,
    OpcNodeset___Xml,
    Oscore,
    Oxps,
    P2pOverlay___Xml,
    P21___Zip,
    Parityfec,
    Passport,
    PatchOpsError___Xml,
    Pdx,
    PemCertificateChain,
    PgpEncrypted,
    PgpKeys,
    PgpSignature,
    Pidf___Xml,
    PidfDiff___Xml,
    Pkcs7Mime,
    Pkcs7Signature,
    Pkcs8,
    Pkcs8Encrypted,
    Pkcs10,
    Pkcs12,
    PkiMessage,
    PkixAttrCert,
    PkixCert,
    Pkixcmp,
    PkixCrl,
    PkixPkipath,
    Pls___Xml,
    PocSettings___Xml,
    PpspTracker___Json,
    Problem___Json,
    Problem___Xml,
    Provenance___Xml,
    Prs_Alvestrand_TitraxSheet,
    Prs_Cww,
    Prs_Cyn,
    Prs_Hpub___Zip,
    Prs_ImpliedDocument___Xml,
    Prs_ImpliedExecutable,
    Prs_ImpliedStructure,
    Prs_Nprend,
    Prs_Plucker,
    Prs_RdfXmlCrypt,
    Prs_Xsf___Xml,
    Pskc___Xml,
    Pvd___Json,
    Qsig,
    Raptorfec,
    Rdap___Json,
    Rdf___Xml,
    Reginfo___Xml,
    RelaxNgCompactSyntax,
    Reputon___Json,
    ResourceLists___Xml,
    ResourceListsDiff___Xml,
    Rfc___Xml,
    Riscos,
    Rlmi___Xml,
    RlsServices___Xml,
    RouteApd___Xml,
    RouteSTsid___Xml,
    RouteUsd___Xml,
    RpkiChecklist,
    RpkiGhostbusters,
    RpkiManifest,
    RpkiPublication,
    RpkiRoa,
    RpkiUpdown,
    Rtf,
    Rtploopback,
    Rtx,
    Samlassertion___Xml,
    Samlmetadata___Xml,
    Sarif___Json,
    SarifExternalProperties___Json,
    Sbe,
    Sbml___Xml,
    Scaip___Xml,
    Scim___Json,
    ScvpCvRequest,
    ScvpCvResponse,
    ScvpVpRequest,
    ScvpVpResponse,
    Sdp,
    Secevent___Jwt,
    Senml___Cbor,
    Senml___Json,
    Senml___Xml,
    SenmlEtch___Cbor,
    SenmlEtch___Json,
    SenmlExi,
    Sensml___Cbor,
    Sensml___Json,
    Sensml___Xml,
    SensmlExi,
    Sep___Xml,
    SepExi,
    SessionInfo,
    SetPayment,
    SetPaymentInitiation,
    SetRegistration,
    SetRegistrationInitiation,
    Sgml,
    SgmlOpenCatalog,
    Shf___Xml,
    Sieve,
    SimpleFilter___Xml,
    SimpleMessageSummary,
    Simplesymbolcontainer,
    Sipc,
    Slate,
    Smil___Xml,
    Smpte336m,
    Soap___Fastinfoset,
    Soap___Xml,
    SparqlQuery,
    SparqlResults___Xml,
    Spdx___Json,
    SpiritsEvent___Xml,
    Sql,
    Srgs,
    Srgs___Xml,
    Sru___Xml,
    Ssml___Xml,
    Stix___Json,
    Swid___Cbor,
    Swid___Xml,
    TampApexUpdate,
    TampApexUpdateConfirm,
    TampCommunityUpdate,
    TampCommunityUpdateConfirm,
    TampError,
    TampSequenceAdjust,
    TampSequenceAdjustConfirm,
    TampStatusQuery,
    TampStatusResponse,
    TampUpdate,
    TampUpdateConfirm,
    Taxii___Json,
    Td___Json,
    Tei___Xml,
    Tetra_isi,
    Thraud___Xml,
    TimestampedData,
    TimestampQuery,
    TimestampReply,
    Tlsrpt___Gzip,
    Tlsrpt___Json,
    Tm___Json,
    Tnauthlist,
    TokenIntrospection___Jwt,
    TrickleIceSdpfrag,
    Trig,
    Ttml___Xml,
    TveTrigger,
    Tzif,
    TzifLeap,
    Ulpfec,
    UrcGrpsheet___Xml,
    UrcRessheet___Xml,
    UrcTargetdesc___Xml,
    UrcUisocketdesc___Xml,
    Vcard___Json,
    Vcard___Xml,
    Vemmi,
    Voicexml___Xml,
    VoucherCms___Json,
    VqRtcpxr,
    Wasm,
    Watcherinfo___Xml,
    WebpushOptions___Json,
    WhoisppQuery,
    WhoisppResponse,
    Widget,
    Wita,
    Wordperfect5_1,
    Wsdl___Xml,
    Wspolicy___Xml,
    WwwFormUrlencoded,
    X400Bp,
    X509CaCert,
    X509CaRaCert,
    X509NextCaCert,
    Xacml___Xml,
    XcapAtt___Xml,
    XcapCaps___Xml,
    XcapDiff___Xml,
    XcapEl___Xml,
    XcapError___Xml,
    XcapNs___Xml,
    XconConferenceInfo___Xml,
    XconConferenceInfoDiff___Xml,
    Xenc___Xml,
    Xfdf,
    Xhtml___Xml,
    Xliff___Xml,
    XmlDtd,
    XmlExternalParsedEntity,
    XmlPatch___Xml,
    Xmpp___Xml,
    Xop___Xml,
    Xslt___Xml,
    Xv___Xml,
    Yang,
    YangData___Cbor,
    YangData___Json,
    YangData___Xml,
    YangPatch___Json,
    YangPatch___Xml,
    Yin___Xml,
    Zlib,
    Zstd,
    _1dInterleavedParityfec,
    _3gpdashQoeReport___Xml,
    _3gpphal___Json,
    _3gpphalforms___Json,
    _3gppIms___Xml,
)
from .presentation import Presentation
from .serialization import (
    InformalSchema,
    Json,
    JsonSchema,
    Schema,
    TextSerialization,
    Toml,
    Xml,
    XmlSchema,
    Yaml,
)
from .spreadsheet import Spreadsheet

from fileformats.text import Javascript  # isort: skip
from fileformats.vendor.openxmlformats_officedocument.application import (  # isort:skip
    Wordprocessingml_Document as MswordX,
)

__all__ = [
    "__version__",
    "Archive",
    "Zip",
    "Bzip",
    "Gzip",
    "Tar",
    "TarGzip",
    "Document",
    "Pdf",
    "Msword",
    "MswordX",
    "Postscript",
    "Schema",
    "JsonSchema",
    "XmlSchema",
    "InformalSchema",
    "TextSerialization",
    "Xml",
    "Json",
    "Yaml",
    "Toml",
    "Dicom",
    "_1dInterleavedParityfec",
    "_3gpdashQoeReport___Xml",
    "_3gpphal___Json",
    "_3gpphalforms___Json",
    "_3gppIms___Xml",
    "A2l",
    "Ace___Cbor",
    "Ace___Json",
    "Activemessage",
    "Activity___Json",
    "Aif___Cbor",
    "Aif___Json",
    "AltoCdni___Json",
    "AltoCdnifilter___Json",
    "AltoCostmap___Json",
    "AltoCostmapfilter___Json",
    "AltoDirectory___Json",
    "AltoEndpointprop___Json",
    "AltoEndpointpropparams___Json",
    "AltoEndpointcost___Json",
    "AltoEndpointcostparams___Json",
    "AltoError___Json",
    "AltoNetworkmapfilter___Json",
    "AltoNetworkmap___Json",
    "AltoPropmap___Json",
    "AltoPropmapparams___Json",
    "AltoUpdatestreamcontrol___Json",
    "AltoUpdatestreamparams___Json",
    "Aml",
    "AndrewInset",
    "Applefile",
    "At___Jwt",
    "Atf",
    "Atfx",
    "Atom___Xml",
    "Atomcat___Xml",
    "Atomdeleted___Xml",
    "Atomicmail",
    "Atomsvc___Xml",
    "AtscDwd___Xml",
    "AtscDynamicEventMessage",
    "AtscHeld___Xml",
    "AtscRdt___Json",
    "AtscRsat___Xml",
    "Atxml",
    "AuthPolicy___Xml",
    "AutomationmlAml___Xml",
    "AutomationmlAmlx___Zip",
    "BacnetXdd___Zip",
    "BatchSmtp",
    "Beep___Xml",
    "Calendar___Json",
    "Calendar___Xml",
    "CallCompletion",
    "Cals_1840",
    "Captive___Json",
    "Cbor",
    "CborSeq",
    "Cccex",
    "Ccmp___Xml",
    "Ccxml___Xml",
    "Cda___Xml",
    "Cdfx___Xml",
    "CdmiCapability",
    "CdmiContainer",
    "CdmiDomain",
    "CdmiObject",
    "CdmiQueue",
    "Cdni",
    "Cea",
    "Cea_2018___Xml",
    "Cellml___Xml",
    "Cfw",
    "City___Json",
    "Clr",
    "ClueInfo___Xml",
    "Clue___Xml",
    "Cms",
    "Cnrp___Xml",
    "CoapGroup___Json",
    "CoapPayload",
    "Commonground",
    "ConciseProblemDetails___Cbor",
    "ConferenceInfo___Xml",
    "Cpl___Xml",
    "Cose",
    "CoseKey",
    "CoseKeySet",
    "CoseX509",
    "Csrattrs",
    "Csta___Xml",
    "Cstadata___Xml",
    "Csvm___Json",
    "Cwl",
    "Cwl___Json",
    "Cwt",
    "Cybercash",
    "Dash___Xml",
    "DashPatch___Xml",
    "Dashdelta",
    "Davmount___Xml",
    "DcaRft",
    "Dcd",
    "DecDx",
    "DialogInfo___Xml",
    "Dicom___Json",
    "Dicom___Xml",
    "Dii",
    "Dit",
    "Dns",
    "Dns___Json",
    "DnsMessage",
    "Dots___Cbor",
    "Dpop___Jwt",
    "Dskpp___Xml",
    "Dssc___Der",
    "Dssc___Xml",
    "Dvcs",
    "EdiConsent",
    "Edifact",
    "EdiX12",
    "Efi",
    "Elm___Json",
    "Elm___Xml",
    "Emergencycalldata_Cap___Xml",
    "Emergencycalldata_Comment___Xml",
    "Emergencycalldata_Control___Xml",
    "Emergencycalldata_Deviceinfo___Xml",
    "Emergencycalldata_Ecall_Msd",
    "Emergencycalldata_Legacyesn___Json",
    "Emergencycalldata_Providerinfo___Xml",
    "Emergencycalldata_Serviceinfo___Xml",
    "Emergencycalldata_Subscriberinfo___Xml",
    "Emergencycalldata_Veds___Xml",
    "Emma___Xml",
    "Emotionml___Xml",
    "Encaprtp",
    "Epp___Xml",
    "Epub___Zip",
    "Eshop",
    "Exi",
    "ExpectCtReport___Json",
    "Express",
    "Fastinfoset",
    "Fastsoap",
    "Fdf",
    "Fdt___Xml",
    "Fhir___Json",
    "Fhir___Xml",
    "Fits",
    "Flexfec",
    "FrameworkAttributes___Xml",
    "Geo___Json",
    "Geo___JsonSeq",
    "Geopackage___Sqlite3",
    "Geoxacml___Xml",
    "GltfBuffer",
    "Gml___Xml",
    "H224",
    "Held___Xml",
    "Hl7v2___Xml",
    "Http",
    "Hyperstudio",
    "IbeKeyRequest___Xml",
    "IbePkgReply___Xml",
    "IbePpData",
    "Iges",
    "ImIscomposing___Xml",
    "Index",
    "Index_Cmd",
    "Index_Obj",
    "Index_Response",
    "Index_Vnd",
    "Inkml___Xml",
    "Iotp",
    "Ipfix",
    "Ipp",
    "Isup",
    "Its___Xml",
    "JavaArchive",
    "Jf2feed___Json",
    "Jose",
    "Jose___Json",
    "Jrd___Json",
    "Jscalendar___Json",
    "JsonPatch___Json",
    "JsonSeq",
    "Jwk___Json",
    "JwkSet___Json",
    "Jwt",
    "KpmlRequest___Xml",
    "KpmlResponse___Xml",
    "Ld___Json",
    "Lgr___Xml",
    "LinkFormat",
    "Linkset",
    "Linkset___Json",
    "LoadControl___Xml",
    "Logout___Jwt",
    "Lost___Xml",
    "Lostsync___Xml",
    "Lpf___Zip",
    "Lxf",
    "MacBinhex40",
    "Macwriteii",
    "Mads___Xml",
    "Manifest___Json",
    "Marc",
    "Marcxml___Xml",
    "Mathematica",
    "Mathml___Xml",
    "MathmlContent___Xml",
    "MathmlPresentation___Xml",
    "MbmsAssociatedProcedureDescription___Xml",
    "MbmsDeregister___Xml",
    "MbmsEnvelope___Xml",
    "MbmsMskResponse___Xml",
    "MbmsMsk___Xml",
    "MbmsProtectionDescription___Xml",
    "MbmsReceptionReport___Xml",
    "MbmsRegisterResponse___Xml",
    "MbmsRegister___Xml",
    "MbmsSchedule___Xml",
    "MbmsUserServiceDescription___Xml",
    "Mbox",
    "MediaControl___Xml",
    "MediaPolicyDataset___Xml",
    "Mediaservercontrol___Xml",
    "MergePatch___Json",
    "Metalink4___Xml",
    "Mets___Xml",
    "Mf4",
    "Mikey",
    "Mipc",
    "MissingBlocks___CborSeq",
    "MmtAei___Xml",
    "MmtUsd___Xml",
    "Mods___Xml",
    "MossKeys",
    "MossSignature",
    "MosskeyData",
    "MosskeyRequest",
    "Mp21",
    "Mp4",
    "Mpeg4Generic",
    "Mpeg4Iod",
    "Mpeg4IodXmt",
    "MrbConsumer___Xml",
    "MrbPublish___Xml",
    "MscIvr___Xml",
    "MscMixer___Xml",
    "Mud___Json",
    "MultipartCore",
    "Mxf",
    "NQuads",
    "NTriples",
    "Nasdata",
    "NewsCheckgroups",
    "NewsGroupinfo",
    "NewsTransmission",
    "Nlsml___Xml",
    "Node",
    "Nss",
    "OauthAuthzReq___Jwt",
    "ObliviousDnsMessage",
    "OcspRequest",
    "OcspResponse",
    "OctetStream",
    "Oda",
    "Odm___Xml",
    "Odx",
    "OebpsPackage___Xml",
    "Ogg",
    "OhttpKeys",
    "OpcNodeset___Xml",
    "Oscore",
    "Oxps",
    "P21",
    "P21___Zip",
    "P2pOverlay___Xml",
    "Parityfec",
    "Passport",
    "PatchOpsError___Xml",
    "Pdx",
    "PemCertificateChain",
    "PgpEncrypted",
    "PgpKeys",
    "PgpSignature",
    "PidfDiff___Xml",
    "Pidf___Xml",
    "Pkcs10",
    "Pkcs7Mime",
    "Pkcs7Signature",
    "Pkcs8",
    "Pkcs8Encrypted",
    "Pkcs12",
    "PkixAttrCert",
    "PkixCert",
    "PkixCrl",
    "PkixPkipath",
    "Pkixcmp",
    "Pls___Xml",
    "PocSettings___Xml",
    "PpspTracker___Json",
    "Presentation",
    "Problem___Json",
    "Problem___Xml",
    "Provenance___Xml",
    "Prs_Alvestrand_TitraxSheet",
    "Prs_Cww",
    "Prs_Cyn",
    "Prs_Hpub___Zip",
    "Prs_ImpliedDocument___Xml",
    "Prs_ImpliedExecutable",
    "Prs_ImpliedStructure",
    "Prs_Nprend",
    "Prs_Plucker",
    "Prs_RdfXmlCrypt",
    "Prs_Xsf___Xml",
    "Pskc___Xml",
    "Pvd___Json",
    "Rdf___Xml",
    "RouteApd___Xml",
    "RouteSTsid___Xml",
    "RouteUsd___Xml",
    "Qsig",
    "Raptorfec",
    "Rdap___Json",
    "Reginfo___Xml",
    "RelaxNgCompactSyntax",
    "Reputon___Json",
    "ResourceListsDiff___Xml",
    "ResourceLists___Xml",
    "Rfc___Xml",
    "Riscos",
    "Rlmi___Xml",
    "RlsServices___Xml",
    "RpkiChecklist",
    "RpkiGhostbusters",
    "RpkiManifest",
    "RpkiPublication",
    "RpkiRoa",
    "RpkiUpdown",
    "Rtf",
    "Rtploopback",
    "Rtx",
    "Samlassertion___Xml",
    "Samlmetadata___Xml",
    "SarifExternalProperties___Json",
    "Sarif___Json",
    "Sbe",
    "Sbml___Xml",
    "Scaip___Xml",
    "Scim___Json",
    "ScvpCvRequest",
    "ScvpCvResponse",
    "ScvpVpRequest",
    "ScvpVpResponse",
    "Sdp",
    "Secevent___Jwt",
    "SenmlEtch___Cbor",
    "SenmlEtch___Json",
    "SenmlExi",
    "Senml___Cbor",
    "Senml___Json",
    "Senml___Xml",
    "SensmlExi",
    "Sensml___Cbor",
    "Sensml___Json",
    "Sensml___Xml",
    "SepExi",
    "Sep___Xml",
    "SessionInfo",
    "SetPayment",
    "SetPaymentInitiation",
    "SetRegistration",
    "SetRegistrationInitiation",
    "Sgml",
    "SgmlOpenCatalog",
    "Shf___Xml",
    "Sieve",
    "SimpleFilter___Xml",
    "SimpleMessageSummary",
    "Simplesymbolcontainer",
    "Sipc",
    "Slate",
    "Smil___Xml",
    "Smpte336m",
    "Soap___Fastinfoset",
    "Soap___Xml",
    "SparqlQuery",
    "Spdx___Json",
    "SparqlResults___Xml",
    "SpiritsEvent___Xml",
    "Spreadsheet",
    "Sql",
    "Srgs",
    "Srgs___Xml",
    "Sru___Xml",
    "Ssml___Xml",
    "Stix___Json",
    "Swid___Cbor",
    "Swid___Xml",
    "TampApexUpdate",
    "TampApexUpdateConfirm",
    "TampCommunityUpdate",
    "TampCommunityUpdateConfirm",
    "TampError",
    "TampSequenceAdjust",
    "TampSequenceAdjustConfirm",
    "TampStatusQuery",
    "TampStatusResponse",
    "TampUpdate",
    "TampUpdateConfirm",
    "Taxii___Json",
    "Td___Json",
    "Tei___Xml",
    "Tetra_isi",
    "Thraud___Xml",
    "TimestampQuery",
    "TimestampReply",
    "TimestampedData",
    "Tlsrpt___Gzip",
    "Tlsrpt___Json",
    "Tm___Json",
    "Tnauthlist",
    "TokenIntrospection___Jwt",
    "TrickleIceSdpfrag",
    "Trig",
    "Ttml___Xml",
    "TveTrigger",
    "Tzif",
    "TzifLeap",
    "Ulpfec",
    "UrcGrpsheet___Xml",
    "UrcRessheet___Xml",
    "UrcTargetdesc___Xml",
    "UrcUisocketdesc___Xml",
    "Vcard___Json",
    "Vcard___Xml",
    "Vemmi",
    "Voicexml___Xml",
    "VoucherCms___Json",
    "VqRtcpxr",
    "Wasm",
    "Watcherinfo___Xml",
    "WebpushOptions___Json",
    "WhoisppQuery",
    "WhoisppResponse",
    "Widget",
    "Wita",
    "Wordperfect5_1",
    "Wsdl___Xml",
    "Wspolicy___Xml",
    "PkiMessage",
    "WwwFormUrlencoded",
    "X509CaCert",
    "X509CaRaCert",
    "X509NextCaCert",
    "X400Bp",
    "Xacml___Xml",
    "XcapAtt___Xml",
    "XcapCaps___Xml",
    "XcapDiff___Xml",
    "XcapEl___Xml",
    "XcapError___Xml",
    "XcapNs___Xml",
    "XconConferenceInfoDiff___Xml",
    "XconConferenceInfo___Xml",
    "Xenc___Xml",
    "Xfdf",
    "Xhtml___Xml",
    "Xliff___Xml",
    "XmlDtd",
    "XmlExternalParsedEntity",
    "XmlPatch___Xml",
    "Xmpp___Xml",
    "Xop___Xml",
    "Xslt___Xml",
    "Xv___Xml",
    "Yang",
    "YangData___Cbor",
    "YangData___Json",
    "YangData___Xml",
    "YangPatch___Json",
    "YangPatch___Xml",
    "Yin___Xml",
    "Zlib",
    "Zstd",
    "Javascript",
]



from abc import abstractproperty

from fileformats.generic import File


class AbstractFile(File):
    """Base class for all medical imaging data including pre-image raw data and
    associated data"""

    @abstractproperty
    def a_property(self) -> str:
        raise NotImplementedError


class AbstractSubclass(AbstractFile):
    pass


class ConcreteClass(AbstractFile):
    @property
    def a_property(self) -> str:
        return "Concrete implementation of a_property"


class AnotherConcreteClass(AbstractSubclass):
    @property
    def a_property(self) -> str:
        return "Another concrete implementation of a_property"


class ConvertibleToFile(File):
    """A class that can be converted from a file"""

    pass
